Package: PRECAST
Type: Package
Title: Embedding and Clustering with Alignment for Spatial Omics
        Datasets
Version: 1.9
Date: 2026-06-05
Authors@R: 
    c(person(given = "Wei",
             family = "Liu",
             role = c("aut", "cre"),
             email = "liuweideng@gmail.com"),
      person(given = "Yi",
             family = "Yang",
             role = c("aut")),
      person(given = "Jin",
             family = "Liu",  
             role = "aut"))
Author: Wei Liu [aut, cre],
  Yi Yang [aut],
  Jin Liu [aut]
Maintainer: Wei Liu <liuweideng@gmail.com>
Description: An efficient data integration method is provided for multiple spatial transcriptomics data with non-cluster-relevant effects such as the complex batch effects. It unifies spatial factor analysis simultaneously with spatial clustering and embedding alignment, requiring only partially shared cell/domain clusters across datasets. More details can be referred to Wei Liu, et al. (2023) <doi:10.1038/s41467-023-35947-w>.
License: GPL-3
Depends: parallel, gtools, R (>= 4.0.0)
Imports: MASS, Matrix, mclust, methods, purrr, utils, Seurat, irlba,
        cowplot, patchwork, scater, pbapply, ggthemes, dplyr, ggplot2,
        stats, DR.SC, scales, ggpubr, graphics, colorspace, Rcpp (>=
        1.0.5)
LazyData: true
URL: https://github.com/feiyoung/PRECAST
BugReports: https://github.com/feiyoung/PRECAST/issues
Suggests: knitr, rmarkdown
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-05 09:29:56 UTC; 10297
Repository: CRAN
Date/Publication: 2026-06-05 10:10:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-06-05 13:19:04 UTC; unix
Archs: PRECAST.so.dSYM
